
load.lib<-c("betareg", "splines")

## If your R is not loaded with these two functions, it can do the check, and load it for you! 

install.lib<-load.lib[!load.lib %in% installed.packages()]
for(lib in install.lib) install.packages(lib,dependences=TRUE)
sapply(load.lib,require,character=TRUE)


library(MASS) 
library(betareg)
library(splines)


#########################################################################
##################Quasi-likelihood estimation for proportional data######
#########################################################################

#####input: 
    # x:covariates
    # y:response
    # fa: offset parameter

#####output

    # beta: estimate
    # sd  : standard error estimate


betareg_quasi<-function(x,y,fa=0){
beta=coefficients(lm(y~0+x))
diff=1;iter=0
while (diff>0.0001 & iter<40)
{ 
  iter=iter+1
  bb=x%*%beta+fa
  bb[abs(bb)>100]=0

  p=exp(bb)/(1+exp(bb))
  w=diag(as.vector(p*(1-p)))
  beta0=beta
  beta=beta+solve(t(x)%*%w%*%x+1e-6*diag(dim(x)[2]))%*%(t(x)%*%(y-p))
  diff=max(abs(beta-beta0))
}
  wd<-solve(t(x)%*%w%*%x)%*%(t(x)%*%diag(as.vector(y-p)^2)%*%x)%*%solve(t(x)%*%w%*%x)
  sd0<-diag(wd)^0.5

  list(beta=beta,sd=sd0)
}

#############################################################################
################### QIF estimate for proportional data#######################
#############################################################################

#######input
      # x:    covariates or design matrix
      # y:    response
      # beta: initial estimate
      # nk:   number of observations for each subject
      # type: working correlation
              # cs for compound symmetry 
              # ar for first-order autoregressive
              # wi for working independence 

#######output
      # beta: estimate
      # sd_est: standard error estimate

qif_prop1<-function(x,y,beta,nk,type){

dx=dim(x)[2];nsub=length(nk)

n<-sum(nk);cn=c(0,cumsum(nk))

maxit = 400;  


betadiff = 1; iteration = 0;
betanew = beta;Q=1e+6;w=1;

while (betadiff > 1e-4 & iteration < maxit){

    iteration = iteration + 1;
    beta = betanew;
    arsumg = 0; arsumc = 0; arsumgfirstdev = 0;
    Q0=Q;
    
    
for (i in 1:nsub){

    xi = x[(cn[i]+1):cn[i+1],];if (nk[i]==1) xi=matrix(xi,nk[i],dx,byrow=T)
    yi = y[(cn[i]+1):cn[i+1],];if (nk[i]==1) yi=matrix(yi,nk[i],1,byrow=T)                

    eta=xi%*%beta
    mui =exp(eta)/(1+exp(eta))
    vmui=diag(as.vector(mui*(1-mui)),nk[i],nk[i])#diag(nk[i])
    dvmui=diag(as.vector(1/mui/(1-mui)),nk[i],nk[i])
    ei = yi-mui

    if (type=="cs"){
      
      m1 = matrix(0, nk[i],nk[i]);
   
      if (nk[i]>1)  m1 = matrix(1,nk[i],nk[i]) - diag(nk[i])
      
      gi = rbind(t(xi)%*%ei, t(xi)%*%m1%*%ei)/nsub;
      di = -rbind(t(xi)%*%vmui%*%xi, t(xi)%*%vmui%*%m1%*%xi)/nsub;

    }else if(type=="ar"){

      m1 = m2 = matrix(0, nk[i],nk[i]);
    
      if (nk[i]>1) {
        m1[1,2] = 1;
        m1[nk[i],nk[i]-1] = 1;
     

      }

      if (nk[i]>2){      
      
         for (j in 2: (nk[i]-1)){
             m1[j,j-1] = 1;
             m1[j,j+1] = 1;
         }
         m2[1,1]=1;m2[nk[i],nk[i]]=1

      }

      gi = rbind(t(xi)%*%ei, t(xi)%*%m1%*%ei, t(xi)%*%m2%*%ei)/nsub;
      di = -rbind(t(xi)%*%vmui%*%xi, t(xi)%*%vmui%*%m1%*%xi, t(xi)%*%vmui%*%m2%*%xi)/nsub;



    }else{

      gi = t(xi)%*%ei/nsub;
      di = -t(xi)%*%vmui%*%xi/nsub;
     
    }

        arsumg = arsumg + gi;
        arsumc = arsumc + gi%*%t(gi);
        arsumgfirstdev = arsumgfirstdev + di;
      
    }

     #arsumc=arsumc*nsub-arsumg%*%t(arsumg);
     arcinv = ginv(arsumc);
    
    Q = t(arsumg)%*%arcinv%*%arsumg;
    arqif1dev = t(arsumgfirstdev)%*%arcinv%*%arsumg;
    arqif2dev = t(arsumgfirstdev)%*%arcinv%*%arsumgfirstdev;
    invarqif2dev = ginv(arqif2dev);
    betanew = beta - w*invarqif2dev%*%arqif1dev;
    betadiff = sum(abs(betanew-beta));  
    #if (max(abs(arqif1dev))<1e-6)  break
    #if (abs(Q-Q0)<1e-6) break 
   
    #w=w/2

}


#sd_est=diag(ginv(t(arsumgfirstdev)%*%arcinv%*%arsumgfirstdev))^0.5;#diag(invarqif2dev)^0.5;

H=invarqif2dev;sd_est=diag(H)^0.5;

list(beta=betanew,sd_est=sd_est,Q=Q,diff=betadiff,H=H)

}





